@@ -2,8 +2,7 @@ import asyncio
 import logging
 from contextlib import suppress
 
-from async_dns.core import REQUEST, DNSMessage, RandId, Record, types
-from pyroute2 import IPRoute
+from dns import exception, message, rdatatype
 
 from .network import SystemNetworkData
 
@@ -19,13 +18,6 @@ DNS_PORT = 53
 _LOGGER = logging.getLogger(__name__)
 
 
-def ip_to_ptr(ip_address):
-    """Convert an ip string to a PTR."""
-    ipl = ip_address.split(".")
-    ipl.reverse()
-    return f"{'.'.join(ipl)}.in-addr.arpa"
-
-
 def short_hostname(hostname):
     """The first part of the hostname."""
     return hostname.split(".")[0]
@@ -33,12 +25,14 @@ def short_hostname(hostname):
 
 def dns_message_short_hostname(dns_message):
     """Get the short hostname from a dns message."""
-    if not isinstance(dns_message, DNSMessage):
+    if not isinstance(dns_message, message.Message) or not dns_message.answer:
         return None
-    record = dns_message.get_record((types.PTR,))
-    if record is None:
-        return None
-    return short_hostname(record)
+    for answer in dns_message.answer:
+        if answer.rdtype != rdatatype.PTR:
+            continue
+        for item in answer.items:
+            return short_hostname(item.target.to_text())
+    return None
 
 
 class PTRResolver:
@@ -48,7 +42,7 @@ class PTRResolver:
         """Init protocol for a destination."""
         self.destination = destination
         self.responses = {}
-        self.send_qid = None
+        self.send_id = None
         self.responded = asyncio.Event()
         self.error = None
 
@@ -63,9 +57,12 @@ class PTRResolver:
         """Response recieved."""
         if addr != self.destination:
             return
-        msg = DNSMessage.parse(data)
-        self.responses[msg.qid] = msg
-        if msg.qid == self.send_qid:
+        try:
+            msg = message.from_wire(data)
+        except exception.DNSException:
+            return
+        self.responses[msg.id] = msg
+        if msg.id == self.send_id:
             self.responded.set()
 
     def error_received(self, exc):
@@ -76,8 +73,8 @@ class PTRResolver:
     async def send_query(self, query):
         """Send a query and wait for a response."""
         self.responded.clear()
-        self.send_qid = query.qid
-        self.transport.sendto(query.pack(), self.destination)
+        self.send_id = query.id
+        self.transport.sendto(query.to_wire(), self.destination)
         await self.responded.wait()
         if self.error:
             raise self.error
@@ -97,23 +94,18 @@ async def async_query_for_ptrs(nameserver, ips_to_lookup):
         transport.close()
 
 
-def async_generate_ptr_query(rand_id, ip):
+def async_generate_ptr_query(ip):
     """Generate a ptr query with the next random id."""
-    req = DNSMessage(qr=REQUEST)
-    req.qd = [Record(REQUEST, ip_to_ptr(str(ip)), types.PTR)]
-    req.qid = rand_id.get()
-    rand_id.put(req.qid)
-    return req
+    return message.make_query(ip.reverse_pointer, rdatatype.PTR)
 
 
 async def async_query_for_ptr_with_proto(protocol, ips_to_lookup):
     """Send and receiver the PTR queries."""
     time_outs = 0
     query_for_ip = {}
-    rand_id = RandId()
     for ip in ips_to_lookup:
-        req = async_generate_ptr_query(rand_id, ip)
-        query_for_ip[ip] = req.qid
+        req = async_generate_ptr_query(ip)
+        query_for_ip[ip] = req.id
         try:
             await asyncio.wait_for(
                 protocol.send_query(req), timeout=DNS_RESPONSE_TIMEOUT
@@ -135,6 +127,8 @@ class DiscoverHosts:
         """Init the discovery hosts."""
         self.ip_route = None
         with suppress(Exception):
+            from pyroute2 import IPRoute  # pylint: disable=import-outside-toplevel
+
             self.ip_route = IPRoute()
 
     async def async_discover(self):
@@ -176,17 +170,17 @@ class DiscoverHosts:
                     sys_network_data.network,
                 )
                 break
-            ips.append(str(host))
+            ips.append(host)
 
         hostnames = {}
         for nameserver in all_nameservers:
-            ips_to_lookup = [ip for ip in ips if ip not in hostnames]
+            ips_to_lookup = [ip for ip in ips if str(ip) not in hostnames]
             results = await async_query_for_ptrs(nameserver, ips_to_lookup)
             for idx, ip in enumerate(ips_to_lookup):
                 short_host = dns_message_short_hostname(results[idx])
                 if short_host is None:
                     continue
-                hostnames[ip] = short_host
+                hostnames[str(ip)] = short_host
             if hostnames:
                 # As soon as we have a responsive nameserver, there
                 # is no need to query additional fallbacks
